@@ -4,6 +4,7 @@ Building the index with pyspark.
 import math
 import multiprocessing
 import os
+import logging
 from functools import partial
 from multiprocessing.pool import ThreadPool
 from tempfile import TemporaryDirectory
@@ -22,6 +23,8 @@ from autofaiss.readers.embeddings_iterators import get_matrix_reader, make_path_
 from autofaiss.utils.cast import cast_memory_to_bytes
 from autofaiss.utils.decorators import Timeit
 
+logger = logging.getLogger("autofaiss")
+
 
 def _yield_embeddings_batch(
     embeddings_paths: List[str],
@@ -162,7 +165,7 @@ def _get_pyspark_active_session():
     # pylint: disable=protected-access
     ss: Optional[pyspark.sql.SparkSession] = pyspark.sql.SparkSession._instantiatedSession  # mypy: ignore
     if ss is None:
-        print("No pyspark session found, creating a new one!")
+        logger.info("No pyspark session found, creating a new one!")
         ss = (
             pyspark.sql.SparkSession.builder.config("spark.driver.memory", "16G")
             .master("local[1]")
@@ -343,7 +346,9 @@ def run(
         # Merge indices in two stages
         # stage1: each executor merges a batch of indices and saves the merged index to stage2 folder
         rdd.foreach(
-            lambda x: _merge_index(small_indices_folder=temporary_indices_folder, batch_id=x[0], start=x[1], end=x[2],)  # type: ignore
+            lambda x: _merge_index(
+                small_indices_folder=temporary_indices_folder, batch_id=x[0], start=x[1], end=x[2],
+            )  # type: ignore
         )
         # stage2: driver merges the indices generated from stage1
         merged_index = _merge_index(_get_stage2_folder(temporary_indices_folder))
