@@ -248,7 +248,7 @@ class MemEfficientFlatIndex(FaissIndexWrapper):
         offset = 0
 
         # For each batch
-        for emb_array in read_embeddings(self.prod_emb_path, batch_size, verbose=True):
+        for emb_array, _ in read_embeddings(self.prod_emb_path, batch_size, verbose=True):
             # for i in trange(0, self.prod_emb.shape[0], batch_size):
 
             # instanciate a Flat index
